/* nxsmbfax.c [version 1.2] -- send fax from Windows to NeXT fax system via SAMBA.

This is a "print command" program for SAMBA for sending
faxes from Windows applications through a faxmodem attached to a NeXT.

Briefly, from any Windows app., you simply print to "Fax" printer,
this program makes a call-back to a pop-up panel application
(respond.exe) running on Windows, and collects info needed for
faxing (e.g., fax number, fax receiver's name, etc.).  It then
inserts this information into the PS file, and exec's lpr to queue
the job for faxing.  You can delay fax transmissions by
specifying a number of hours to delay.  This is not as flexible as
exact time specification allowd by the NeXT Fax panel, but should do
fine for most.

Requires working fax system such as NXFax on NEXTSTEP/OPENSTEP.

COMPILE:
Single arch compile:
cc -Wall -s -o nxsmbfax nxsmbfax.c
DEBUG:
cc -Wall -s -DDEBUG -o nxsmbfax nxsmbfax.c

Fat compile (add -arch xxxx as needed):
cc -Wall -s -arch m68k -arch i386 -o nxsmbfax nxsmbfax.c

-------------------------------------------------------------------------
SETUP
*** On Windows side:
Get RESPOND.EXE (freeware) from:
   http://www.boerde.de/~horstf/ or http://relay.boerde.de:82/~horstf/
Auto start RESPOND.EXE via STARTUP folder on Windows95/NT.
Configure RESPOND by a companion app CONFIGURE.EXE.
In particular do this:
Enable delayed transmission field as text area.
Relabel this field as "Number of hours to delay", because
  with nxsmbfax, delay is specified by the number of hours (integer)
  from now to call time.

After SAMBA is running, setup \\server\Fax as Apple LaserWriter NTX
(or if you have NeXT 400dpi Laser printer PPD file, as that).
Do NOT choose a HP or PCL compatible printers, as they generate PJL
junk prepended to real PS code.

*** On NS/OS side:

cp nxsmbfax /usr/local/bin
mkdir /usr/local/lib/fax
cp default.coverinfo /usr/local/lib/fax
(Additionally, create your own Fax cover info file as:
    youruserid.coverinfo [lower case] in the same directory.)

Setup SAMBA (smb.conf) with lines like:
--
; Add to your smb.conf
; %I: IP address
; %s: The name of the faxfile with path
; %U: The samba session setup user
; %m: The NetBIOS hostname

[Fax]
   comment       = Fax Modem faxmodem_fax
   postscript	 = yes
   print command = ( /usr/local/bin/nxsmbfax %I %s faxmodem_fax ) &
   printable     = yes
   writable      = no
   path = /tmp
---


### BUGS:
1. DSC comments are not quite legal, but works with NeXT Fax system.
   In particular %%Pages: 100 has a bogus large value which seems OK.
   %%Page: ? nn comments are not also correct (confuses Preview.app).

2. Stripping PJL junk for HP PCL printers is not in.  It does strip ^D chars.


*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <libc.h>

#define SBUFSIZE	4096	// string buffer size
#define CTRL_D	4
#define DEFPORT	5555		// port of RESPOND.EXE running on Windows

static char *coverDir = "/usr/local/lib/fax";

char *strlwr( char *string )
{
char *strptr, ch;
	strptr = string;
	while( (ch = *strptr) != '\0' ) {
		if( isupper(ch))
		     *strptr = tolower(ch);
		strptr++;
	}
	return(string);
}


/* Escapes '\', '(', and ')' for PostScript string.
   Escaped string must fit within allocated string space of
   the input string.
*/
char *escape_psstring(char *string)
{
char *ptr, *ptr2, tempbuf[SBUFSIZE*2], ch;
    ptr = string;
    ptr2 = tempbuf;
    while( (ch = *ptr++) != '\0') {
	if( (ch == '\\') || (ch == '(') || (ch == ')') ) {
	    *ptr2++ = '\\';
	    *ptr2++ = ch;
	}
	else
	    *ptr2++ = ch;
    }
    *ptr2 = '\0';
    tempbuf[SBUFSIZE-2] = '\0';		// make sure it won't go over limit
    strncpy(string, tempbuf, SBUFSIZE-1);
    return(string);
}

void main(int argc, char **argv)
{
int c;
FILE *fp, *fpin, *fpout;
struct in_addr defaddr;
struct hostent *hp, def;
struct sockaddr_in sin;
int s, port=DEFPORT;
char *pptr;
char host[256];
char modemName[32];
char *alist[1];
char buf[SBUFSIZE];
char phonenum[64],userid[32],receiverName[256],senderName[256],timeStr[64],timeStrNow[64];
char comdbuf[1024];
char coverinfopath[1025];
char psfilepath[1025];
char pagecountpath[1025];
u_long inet_addr();
int delay =0, linecnt = 0, pagecount = -9999;
time_t ltime=0, ltimenow=0;
int ypos=720, ybump=30;

	if(argc != 4) {
	    fprintf(stderr, "Usage: %s IPadd psfilefullpath modemname_fax\n", argv[0]);
	    exit(1);
	}

	strncpy(modemName, argv[3], 30);
	strncpy(psfilepath, argv[2], 1024);
	strncpy(pagecountpath, argv[2], 1024);
	strcat(psfilepath, "~");
	strcat(pagecountpath, "~pcnt");
	/* spooled PS file and temporary output file */
	if((fpin = fopen(argv[2], "r")) == NULL) {
		fprintf(stderr, "nxsmbfax: can't open PS file for faxing: %s\n", argv[2]);
		exit(1);
	}
	if((fpout = fopen(psfilepath, "w")) == NULL) {
		fprintf(stderr, "nxsmbfax: can't open temp file for faxing: %s\n", psfilepath);
		exit(1);
	}

#ifndef DEBUG
	/* Make a call-back to Windows PC via RESPOND.EXE for Fax info */
	if (!(hp = gethostbyname(argv[1]))) {
		defaddr.s_addr = inet_addr(argv[1]);
		if (defaddr.s_addr == -1) {
			fprintf(stderr, "nxsmbfax: unknown host: %s\n", host);
			exit(1);
		}
		def.h_name = argv[1];
		def.h_addr_list = alist;
		def.h_addr = (char *)&defaddr;
		def.h_length = sizeof(struct in_addr);
		def.h_addrtype = AF_INET;
		def.h_aliases = 0;
		hp = &def;
	}
	sin.sin_family = hp->h_addrtype;
	bcopy(hp->h_addr, (char *)&sin.sin_addr, hp->h_length);
	sin.sin_port = htons(port);
	if ((s = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0) {
		perror("nxsmbfax: socket");
		exit(1);
	}
	/* have network connection; identify the host connected with */
	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		perror("nxsmbfax: connect");
		close(s);
		exit(1);
	}

	/* Read from RESPOND.EXE on Windows for fax info */
	/* char phonenum[64],userid[32],receiverName[256],senderName[256],timeStr[64]; */
	linecnt = 0;
	if ((fp = fdopen(s, "r"))) {
	    while (fgets(buf, 510, fp) != NULL) {
		switch(linecnt) {
		    case 0: sscanf(buf, "%[^'\r']", phonenum); linecnt++; break;
		    case 1: sscanf(buf, "%[^'\r']", userid); linecnt++; break;
		    case 2: sscanf(buf, "%[^'\r']", receiverName); linecnt++; break;
		    case 3: sscanf(buf, "%[^'\r']", senderName); linecnt++; break;
		    case 4: sscanf(buf, "%[^'\r']", timeStr); linecnt++; break;
		    default: break;
		} /* end switch() */
	    } /* end while() */
	    fclose(fp);
	}
	if(linecnt < 2) {
	    /* RESPOND.EXE dialog was cancelled */
	    fprintf(stderr, "Fax cancelled\n");
	    fclose(fpin);
	    fclose(fpout);
	    remove(argv[2]);
	    remove(psfilepath);
	    exit(1);
	}
#else
	/* debugging without RESPOND.EXE dialog */
	linecnt = 5;
	strcpy(phonenum, "9,1 415 123 4567");
	strcpy(userid, "anybody");
	strcpy(receiverName, "Dr. Fax Receiver");
	strcpy(senderName, "Mr. Fax SenderName");
	strcpy(timeStr, "3");
#endif

	/* Determine # of pages by relying on the last "%%Pages: nn"  -- this may fail. */
	/* grep '^%%Pages: [0123456789]' %s | awk '{print $2}'  */
	sprintf(comdbuf, "grep '^%%%%Pages: [0123456789]' %s | awk '{print $2}' | tail -1 > %s ",
				 argv[2], pagecountpath);
	system(comdbuf);
	if((fp = fopen(pagecountpath, "r")) != NULL) {
	    fscanf(fp, "%d", &pagecount);
	    fclose(fp);
	}
#ifdef DEBUG
	fprintf(stderr, "Page count (original): %d\n", pagecount);
#else
	remove(pagecountpath);
#endif
	pagecount++;		/* increment for cover page */

	/* Open Fax Cover Sheet info file, first as userid, then for default file. */
	strlwr(userid);	/* lower case */
	sprintf(coverinfopath, "%s/%s.coverinfo", coverDir, userid);
	if((fp = fopen(coverinfopath, "r")) == NULL) {
	    sprintf(coverinfopath, "%s/default.coverinfo", coverDir);
	    fp = fopen(coverinfopath, "r");
	}

	/* Write Fax DSC comment for NeXT Fax system to temporary file */
	/* char phonenum[64],userid[32],receiverName[256],senderName[256],timeStr[64]; */
	fprintf(fpout, "%%!PS-Adobe-3.0\n");
	fprintf(fpout, "%%%%Title: Fax from a Windows application via network (Samba)\n");
	fprintf(fpout, "%%%%NXFaxVersion: 300\n");
	fprintf(fpout, "%%%%NXFaxNumber: %s\n", phonenum);
	fprintf(fpout, "%%%%NXFaxTo: %s\n", receiverName);
	fprintf(fpout, "%%%%NXFaxNotify: YES\n");
	fprintf(fpout, "%%%%NXFaxHires: YES\n");	/* This could be customized */
	fprintf(fpout, "%%%%NXFaxTrim: NO\n");	/* This could be customized */

	time(&ltimenow);
	delay = atoi(timeStr);
	if(linecnt < 5 || delay <= 0) {
	    /* no delay */
	    fprintf(fpout, "\n\n");
	}
	else {
	    /* do time delay stuff */
	    ltime = ltimenow + 3600 * delay;	/* delay by this many hours */
	    strftime(timeStr, 60, "%Y%m%d%H%M", localtime(&ltime));
	    fprintf(fpout, "%%%%NXDeferredUntil: %s\n\n", timeStr);
	}

	/* Do FAX COVER SHEET */
	fprintf(fpout, "%%%%Pages: 100\n");		/* This is not correct, but seems to be OK. */
	fprintf(fpout, "%%%%Page: ? 1\n\n");
	fprintf(fpout, "/_faxcover_save_token save def\n");
	
	fprintf(fpout, "/Times-Bold findfont 24 scalefont setfont\n");
	fprintf(fpout, "190 %d moveto (FAX COVER SHEET) show\n", ypos);
	ypos -= ybump;
	fprintf(fpout, "/Times-Roman findfont 24 scalefont setfont\n");
	if(fp && (fgets(buf, 510, fp) != NULL)) {
	    if((pptr = strchr(buf, '\n')) != NULL) *pptr = '\0'; /* delete new line */
	    fprintf(fpout, "130 %d moveto (%s) show\n", ypos, escape_psstring(buf)); ypos -= ybump;
	}

	fprintf(fpout, "/Times-Roman findfont 18 scalefont setfont\n");
	while (fp && (fgets(buf, 510, fp) != NULL)) {
	    if((pptr = strchr(buf, '\n')) != NULL) *pptr = '\0'; /* delete new line */
	    fprintf(fpout, "130 %d moveto (%s) show\n", ypos, escape_psstring(buf)); ypos -= ybump;
	}
	fclose(fp);
	fprintf(fpout, "2 setlinewidth\n");
	fprintf(fpout, "50 %d moveto 560 %d lineto stroke\n\n", ypos, ypos); ypos -= ybump;	/* horizontal line */
	fprintf(fpout, "/Helvetica findfont 18 scalefont setfont\n");
	fprintf(fpout, "100 %d moveto (To:      %s) show\n", ypos, escape_psstring(receiverName)); ypos -= ybump;
	fprintf(fpout, "100 %d moveto (From:  %s) show\n", ypos, escape_psstring(senderName)); ypos -= ybump;
	strftime(timeStrNow, 60, "%a %B %d, %Y %H:%M (our time)", localtime(&ltimenow));
	fprintf(fpout, "100 %d moveto (Date:   %s) show\n", ypos, timeStrNow); ypos -= ybump;
	if(pagecount > 0) {
	    fprintf(fpout, "100 %d moveto (Number of pages (including this cover):  %d) show\n", ypos, pagecount);
	    ypos -= ybump;
	}
	fprintf(fpout, "showpage\n");
	fprintf(fpout, "clear _faxcover_save_token restore\n");
	fprintf(fpout, "%% /_faxcover_save_token save def\n\n");

	/* Copy the PS file while removing Ctrl-D etc from head of Windows PS file */
	while ((c = getc(fpin)) != EOF) {
		if(c != CTRL_D) putc(c, fpout); /* remove ctrl-D */
	}
	fclose(fpin);
	fclose(fpout);
	if(delay > 0)
	    sprintf(comdbuf, "exec /usr/ucb/lpr -P%s -D %s %s", modemName, timeStr, psfilepath);
	else
	    sprintf(comdbuf, "exec /usr/ucb/lpr -P%s %s", modemName, psfilepath);
#ifdef DEBUG
	fprintf(stderr, "%s\n", comdbuf);
#else
	system(comdbuf);
	remove(psfilepath);
	remove(argv[2]);
#endif
        exit(0);
}

